# Project name:	Meltdown BBS Utilities
# File:		export-echomail.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $row);

print "\nMeltdown BBS Utilities echomail export script";
print "\nversion 1.0 alpha";
print "\n\n";

#################
# Open log file #
#################

if ($log) {
	open(LOG, ">>" . readini("Logging", "logfile")) || die "Can't open log file: $!\n";
}

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

##################################
# Export messages from each area #
##################################

$query = $db->prepare("SELECT echoid, fidotag, aka FROM echos WHERE fidotag != ''");
$query->execute;

while ( $row = $query->fetchrow_hashref ) {
  &export_area(${$row}{'echoid'}, ${$row}{'fidotag'}, ${$row}{'aka'});
}

#######################################
# export_area (echoid, fidotag, aka)  #
# exports messages from a single area #
#######################################

sub export_area {
	my ($echoid, $fidotag, $aka) = @_;
	my $msgfile;
	my $msgcount=0;
	my $dir = readini("Areas", $fidotag);
	if (!($dir)) { return 0; }
	my ($row, $query);
	my ($header, $body);
	my ($akazone, $akanet, $akanode);

	$akazone = $aka;
	$akazone =~ s/\:(.*)//;
	$akanet = $aka;
	$akanet =~ s/^(.*)://;
	$akanet =~ s/\/(.*)$//;
	$akanode = $aka;
	$akanode =~ s/^(.*)\///;

	&writelog("Processing area $fidotag");

        $query = $db->prepare("SELECT msgid, sender, recipient, subject, replyto, FROM_UNIXTIME(datetime, '%d %b %y  %h:%m:%s'), body FROM msgs WHERE echoid = ? AND sent = 0");
	$query->execute($echoid);
	while ( $row = $query->fetchrow_hashref ) {
		# Open next file
		$msgfile = msgname($dir);
	        open(MSG, ">$dir/$msgfile") || die "Can't open file for writing: $!\n";
		# Write header
		$header = pack("a36a36a72a20SSSSSSSSSSSSS",
			${$row}{'sender'},
			${$row}{'recipient'},
			${$row}{'subject'},
			${$row}{'FROM_UNIXTIME(datetime, \'%d %b %y  %h:%m:%s\')'},
			0,
			0,
			$akanode,
			0,
			$akanet,
			0,
			0,
			$akazone,
			0,
			0,
			0,
			384,
			0
		);
		print MSG $header;
					
		# Add kludges
		my ($msgid, $orig_msgid, $reply, $kludges, $query2);
		$msgid = chr(1) . "MSGID: $aka 0" . sprintf("%07x", ${$row}{'msgid'});

		$query2 = $db->prepare("SELECT orig_msgid FROM msgs WHERE msgid=?");
		$query2->execute(${$row}{'replyto'});
		$orig_msgid = $query2->fetchrow_array;

		if($orig_msgid) { 
			$reply = chr(1) . "REPLY: $orig_msgid"; 
		} else {
			$reply = '';
		}

		$kludges = "$msgid\n$reply\n";
		print MSG $kludges;
		
		# Process body
		my $body = decode_entities(${$row}{'body'});
		print MSG $body;
		close MSG;

		# Update database
		$db->do("UPDATE msgs SET sent=1 WHERE msgid='${$row}{'msgid'}'");
		$msgcount++;
		print "  $msgcount messages exported.\r";
	}
	&writelog("  $msgcount messages exported.");
}

close LOG;
close INI;
$db->disconnect;

#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}

#######################################################################
# writelog (message)                                                  #
# prints message on screen and writes it to the log file (if defined) #
#######################################################################

sub writelog {
	print LOG localtime(time) . " $_[0]\n" if $log;
	print "$_[0]\n";
}

#####################################################
# msgname (dir)                                     #
# returns an unused *.MSG filename in directory dir #
#####################################################

sub msgname {
	my $dir = $_[0];
	my ($file, $last);

	opendir MSGDIR, $dir || die "Can't open directory $dir: $!\n";
	my @files = grep { /\.msg$/i } readdir(MSGDIR);
	closedir MSGDIR;

	$last = 0;
	foreach $file (@files) {
		if ($file > $last) { $last = $file; }
	}
	return ($last + 1 . ".msg");
}