# Project name:	Meltdown BBS Utilities
# File:		export-netmail.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $row);

print "\nMeltdown BBS Utilities netmail export script";
print "\nversion 1.0 alpha";
print "\n\n";

#################
# Open log file #
#################

if ($log) {
	open(LOG, ">>" . readini("Logging", "logfile")) || die "Can't open log file: $!\n";
}

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

###################
# Export messages #
###################

my ($header, $body);
my ($fromAddr, $fromZone, $fromNet, $fromNode, $fromPoint, $toAddr, $toZone, $toNet, $toNode, $toPoint);
my ($dir, $msgfile);
my $msgcount=0;

&writelog("Processing Netmail");

# Get system FTN address
$dir = readini("Areas", "Netmail");
if (!($dir)) {
	close LOG;
	close INI;
	$db->disconnect;
	exit;
}

$query = $db->prepare("SELECT config_value FROM config WHERE config_name='nodenumber'");
$query->execute;

$fromAddr = $query->fetchrow_array;
($fromAddr) = split(/ /, $fromAddr);

$fromZone = $fromAddr;
$fromZone =~ s/\:(.*)//;

$fromNet=$fromAddr;
$fromNet =~ s/^(.*)://;
$fromNet =~ s/\/(.*)$//;

$fromNode = $fromAddr;
$fromNode =~ s/^(.*)\///;
$fromNode =~ s/\.(.*)//;

if ($fromAddr =~ m/\./) {
	$fromPoint = $toAddr;
	$fromPoint = s/^(.*)\.//;
}

$query = $db->prepare("SELECT sender, recipient, subject, FROM_UNIXTIME(datetime, '%d %b %y  %h:%m:%s'), sourcenode, destnode, msgid, body FROM netmail WHERE sent = 0 AND destnode != ''");
$query->execute;

while ( $row = $query->fetchrow_hashref ) {
	# Open next file
	$msgfile = msgname($dir);
        open(MSG, ">$dir/$msgfile") || die "Can't open file for writing: $!\n";

	$toAddr = ${$row}{'destnode'};
	$toZone = $toAddr;
	$toZone =~ s/\:(.*)//;

	$toNet = $toAddr;
	$toNet =~ s/^(.*)://;
	$toNet =~ s/\/(.*)$//;

	$toNode = $toAddr;
	$toNode =~ s/^(.*)\///;
	$toNode =~ s/\.(.*)//;

	if ($toAddr =~ m/\./) {
		$toPoint = $toAddr;
		$toPoint = s/^(.*)\.//;
	}

	# Write header
	$header = pack("a36a36a72a20SSSSSSSSSSSSS",
		${$row}{'sender'},
		${$row}{'recipient'},
		${$row}{'subject'},
		${$row}{'FROM_UNIXTIME(datetime, \'%d %b %y  %h:%m:%s\')'},
		0,
		$toNode,
		$fromNode,
		0,
		$fromNet,
		$toNet,
		$toZone,
		$fromZone,
		0,
		0,
		0,
		384,
		0
	);

	print MSG $header;

	# Add kludges
	$fromAddr =~ s/\.(.*)//;
	$toAddr =~ s/\.(.*)//;
	print MSG chr(1) . "INTL $toAddr $fromAddr";
	if ($fromPoint > 0) {
		print MSG chr(1) . "FMPT $fromPoint";
	}
	if ($toPoint > 0) {
		print MSG chr(1) . "TOPT $toPoint";
	}
	my $msgid = chr(1) . "MSGID: $fromAddr 1" . sprintf("%07x", ${$row}{'msgid'}) . "\n";
	print MSG $msgid;


	# Process body
	my $body = decode_entities(${$row}{'body'});
	print MSG $body;
	close MSG;

	# Update database
	$db->do("UPDATE netmail SET sent=1 WHERE msgid='${$row}{'msgid'}'");
	$msgcount++;
	print "  $msgcount messages exported.\r";
}

&writelog("  $msgcount messages exported.");
close LOG;
close INI;
$db->disconnect;

#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}

#######################################################################
# writelog (message)                                                  #
# prints message on screen and writes it to the log file (if defined) #
#######################################################################

sub writelog {
	print LOG localtime(time) . " $_[0]\n" if $log;
	print "$_[0]\n";
}

#####################################################
# msgname (dir)                                     #
# returns an unused *.MSG filename in directory dir #
#####################################################

sub msgname {
	my $dir = $_[0];
	my ($file, $last);

	opendir MSGDIR, $dir || die "Can't open directory $dir: $!\n";
	my @files = grep { /\.msg$/i } readdir(MSGDIR);
	closedir MSGDIR;

	$last = 0;
	foreach $file (@files) {
		if ($file > $last) { $last = $file; }
	}
	return ($last + 1 . ".msg");
}