# Project name:	Meltdown BBS Utilities
# File:		export-txtfile.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;
use Getopt::Std;
our ($opt_a, $opt_t);
getopt('at');

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $row);

print "\nMeltdown BBS Utilities textfile export script";
print "\nversion 1.0 alpha";
print "\n\n";

if (!($opt_a>0) || !($opt_t)) {
	print "Usage: export-txtfile -a area_number -t text_file";
	close INI;
	exit;
}

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

$query = $db->prepare("SELECT name FROM echos WHERE echoid=?");
$query->execute($opt_a);
my $areaname = $query->fetchrow_array;


$query = $db->prepare("SELECT COUNT(*) FROM msgs WHERE echoid=?");
$query->execute($opt_a);
my $totmsgs = $query->fetchrow_array;


$query = $db->prepare("SELECT sender, recipient, subject, FROM_UNIXTIME(datetime), body FROM msgs WHERE echoid=? ORDER BY datetime, msgid");
$query->execute($opt_a);

open TXTFILE, ">$opt_t" || die "Can't open file $opt_t: $!\n";
my $msgcount = 0;
while ( $row = $query->fetchrow_hashref ) {
	$msgcount++;
	print "Exporting message $msgcount of $totmsgs\r";
	my $body = decode_entities(${$row}{body});
	$body =~ s/\r\n/\n/g;
	print TXTFILE "$areaname\nMessage $msgcount of $totmsgs\n";
	print TXTFILE "-" x 80;
	print TXTFILE "\nFrom:\t${$row}{'sender'}";
	print TXTFILE "\nTo:\t${$row}{'recipient'}";
	print TXTFILE "\nDate:\t${$row}{'FROM_UNIXTIME(datetime)'}";
	print TXTFILE "\nSubj:\t${$row}{'subject'}\n";
	print TXTFILE ("-" x 80);
	print TXTFILE "\n$body";
	print TXTFILE "\n\n";
}

close TXTFILE;
close INI;
$db->disconnect;


#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}