# Project name:	Meltdown BBS Utilities
# File:		import-echomail.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $query2, $row);

print "\nMeltdown BBS Utilities echomail import script";
print "\nversion 1.0 alpha";
print "\n\n";

#################
# Open log file #
#################

if ($log) {
	open(LOG, ">>" . readini("Logging", "logfile")) || die "Can't open log file: $!\n";
}

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

##################################
# Import messages into each area #
##################################

$query = $db->prepare("SELECT echoid, fidotag FROM echos WHERE fidotag != ''");
$query->execute;

while ( $row = $query->fetchrow_hashref ) {
  &import_area(${$row}{'echoid'}, ${$row}{'fidotag'});
}

#######################################
# import_area (echoid, fidotag)       #
# imports messages into a single area #
#######################################

sub import_area {
	my ($echoid, $fidotag) = @_;
	my ($msgfile, $header, $body);
	my $dir = readini("Areas", $fidotag);
	if(!($dir)) { return 0; }
	my $msgcount = 0;
	undef $/;

	&writelog("Processing area $fidotag");

	opendir MSGDIR, $dir || die "Can't open directory $dir: $!\n";
	my @files = grep { /\.msg$/i } readdir(MSGDIR);
	closedir MSGDIR;

	foreach $msgfile (@files) {
		open(MSG,"$dir/$msgfile") || die "Can't open file $msgfile in $dir: $!\n";
		read MSG, $header, 190;
		$body = <MSG>;
		close MSG;

		my (
			$from, $to, $subj, $datetime,
			$timesread,
			$destnode, $orignode, $cost,
			$orignet, $destnet, $destzone,
			$origzone, $destpoint, $origpoint,
			$rplyto, $attribute, $nextreply
		) = unpack("a36a36a72a20SSSSSSSSSSSSS", $header);
		$from =~ s/\00//g;
		$to =~ s/\00//g;
		$subj =~ s/\00//g;

		# Convert date to a format that SQL can understand
		# From: "DD MMM YY  HH:MM:SS"
		# To:   "YYYY-MM-DD HH:MM:SS"
		my %months = (
			Jan=>'01', Feb=>'02', Mar=>'03', Apr=>'04', May=>'05', Jun=>'06',
			Jul=>'07', Aug=>'08', Sep=>'09', Oct=>'10', Nov=>'11', Dec=>'12'
		);
		my($d,$mon,$y,$h,$m,$s) = unpack("A2xA3xA2xxA2xA2xA2x", $datetime);
		if ($y > 70) { $y="19" . $y; } else { $y="20" . $y; }
		$mon = $months{$mon};
		my $sqldate = "$y-$mon-$d $h:$m:$s";

		# Process control lines
		my ($msgid, $reply, $replyto);
		$body =~ s/\r/\n/g;
		if ($body =~ m/(\x01MSGID: )([^\n]+)/im) { $msgid = $2; }
		if ($body =~ m/(\x01REPLY: )([^\n]+)/im) { $reply = $2; }
		$body =~ s/\x01(.*)\n?//gim;
		&encode_entities($body, "\"&<>");

		# All right, now import that thingy
		my $insert = "INSERT INTO msgs(echoid, sender, recipient, subject, datetime, sent, body"
			. (($msgid) ? ", orig_msgid" : "")
			. (($reply) ? ", orig_reply" : "")
			. ") VALUES(?,?,?,?,UNIX_TIMESTAMP(?),?,?"
			. (($msgid) ? ",?" : "")
			. (($reply) ? ",?" : "")
			. ")";
		$query2 = $db->prepare($insert);

		if ($msgid && $reply) {
			$query2->execute($echoid, $from, $to, $subj, $sqldate, 1, $body, $msgid, $reply);
		}
		elsif ($msgid) {
			$query2->execute($echoid, $from, $to, $subj, $sqldate, 1, $body, $msgid);
		}
		elsif ($reply) {
			$query2->execute($echoid, $from, $to, $subj, $sqldate, 1, $body, $reply);
		}
		else {
			$query2->execute($echoid, $from, $to, $subj, $sqldate, 1, $body);
		}

		unlink "$dir/$msgfile";
		$msgcount++;
		print "  $msgcount messages imported.\r";
	}
	&writelog("  $msgcount messages imported.");
}

&writelog("Making thread links...");
$query = $db->prepare("SELECT msgid, orig_reply FROM msgs WHERE orig_reply != '' and replyto IS NULL");
$query->execute;

while ( $row = $query->fetchrow_hashref ) {
	$query2 = $db->prepare("SELECT msgid FROM msgs WHERE orig_msgid=?");
	$query2->execute(${$row}{'orig_reply'});
	my $replyto = $query2->fetchrow_array;
	if ($replyto) {
		$query2 = $db->prepare("UPDATE msgs SET replyto=? WHERE msgid=?");
		$query2->execute($replyto, ${$row}{'msgid'});
	} else {
		$query2 = $db->prepare("UPDATE msgs SET orig_reply='' WHERE msgid=?");
		$query2->execute(${$row}{'msgid'});
	}
}

close LOG;
close INI;
$db->disconnect;

#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}

#######################################################################
# writelog (message)                                                  #
# prints message on screen and writes it to the log file (if defined) #
#######################################################################

sub writelog {
	print LOG localtime(time) . " $_[0]\n" if $log;
	print "$_[0]\n";
}
