# Project name:	Meltdown BBS Utilities
# File:		import-netmail.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $query2, $row);

print "\nMeltdown BBS Utilities netmail import script";
print "\nversion 1.0 alpha";
print "\n\n";

#################
# Open log file #
#################

if ($log) {
	open(LOG, ">>" . readini("Logging", "logfile")) || die "Can't open log file: $!\n";
}

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

###################
# Import messages #
###################

my ($header, $body);
my ($myAddr, $myZone);
my ($dir, $msgfile);
my $msgcount=0;

&writelog("Processing Netmail");

# Get system FTN address

$query = $db->prepare("SELECT config_value FROM config WHERE config_name='nodenumber'");
$query->execute;

$myAddr = $query->fetchrow_array;
($myAddr) = split(/ /, $myAddr);

$myZone = $myAddr;
$myZone =~ s/\:(.*)//;

undef $query;

# This code could be useful for future versions
# if we wanna check whether the message is actually addressed to us.
# However, for the script is just going to blindly import *.MSG
#
#$query = $db->prepare("SELECT config_value FROM config WHERE config_name='nodenumber'");
#$query->execute;
#
#$toAddr = $query->fetchrow_array;
#($toAddr) = split(/ /, $toAddr);
#
#$toZone = $toAddr;
#$toZone =~ s/\:(.*)//;
#
#$toNet=$toAddr;
#$toNet =~ s/^(.*)://;
#$toNet =~ s/\/(.*)$//;
#
#$toNode = $fromAddr;
#$toNode =~ s/^(.*)\///;

$dir = readini("Areas", "netmail");
if (!($dir)) {
	close LOG;
	close INI;
	$db->disconnect;
	exit;
}

undef $/;

opendir MSGDIR, $dir || die "Can't open directory $dir: $!\n";
my @files = grep { /\.msg$/i } readdir(MSGDIR);
closedir MSGDIR;

foreach $msgfile (@files) {
	open(MSG,"$dir/$msgfile") || die "Can't open file $msgfile in $dir: $!\n";
	read MSG, $header, 190;
	$body = <MSG>;
	close MSG;
		my (
		$from, $to, $subj, $datetime,
		$timesread,
		$destnode, $orignode, $cost,
		$orignet, $destnet, $destzone,
		$origzone, $destpoint, $origpoint,
		$replyto, $attribute, $nextreply
	) = unpack("a36a36a72a20SSSSSSSSSSSSS", $header);
	$from =~ s/\00//g;
	$to =~ s/\00//g;
	$subj =~ s/\00//g;

	# Convert date to a format that SQL can understand
	# From: "DD MMM YY  HH:MM:SS"
	# To:   "YYYY-MM-DD HH:MM:SS"
	my %months = (
		Jan=>'01', Feb=>'02', Mar=>'03', Apr=>'04', May=>'05', Jun=>'06',
		Jul=>'07', Aug=>'08', Sep=>'09', Oct=>'10', Nov=>'11', Dec=>'12'
	);
	my($d,$mon,$y,$h,$m,$s) = unpack("A2xA3xA2xxA2xA2xA2x", $datetime);
	if ($y > 70) { $y="19" . $y; } else { $y="20" . $y; }
	$mon = $months{$mon};
	my $sqldate = "$y-$mon-$d $h:$m:$s";

	# Process control lines
	my ($intl, $fmpt);
	if ($body =~ m/(\x01INTL )([^\n]+)/im) { $intl = $2; }
	if ($body =~ m/(\x01FMPT )([^\n]+)/im) { $fmpt = $2; }

	if ($intl) {
		$origzone = $intl;
		$origzone =~ s/^(.*?) //;
		$origzone =~ s/\:(.*)//;
	} else {
		$origzone = $myZone;
	}

	my $srcnode = "$origzone:$orignet/$orignode";
	$srcnode .= ".$fmpt" if ($fmpt > 0);

	$body =~ s/\r/\n/g;
	$body =~ s/\x01/@/g;

	&encode_entities($body, "\"&<>");

	# All right, now import that thingy
	$query2 = $db->prepare("INSERT INTO netmail(sender, recipient, subject, datetime, body, sourcenode) VALUES(?,?,?,UNIX_TIMESTAMP(?),?,?)");
	$query2->execute($from, $to, $subj, $sqldate, $body, $srcnode);

	unlink "$dir/$msgfile";
	$msgcount++;
	print "  $msgcount messages imported.\r";
}

&writelog("  $msgcount messages imported.");
close LOG;
close INI;
$db->disconnect;

#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}

#######################################################################
# writelog (message)                                                  #
# prints message on screen and writes it to the log file (if defined) #
#######################################################################

sub writelog {
	print LOG localtime(time) . " $_[0]\n" if $log;
	print "$_[0]\n";
}

#####################################################
# msgname (dir)                                     #
# returns an unused *.MSG filename in directory dir #
#####################################################

sub msgname {
	my $dir = $_[0];
	my ($file, $last);

	opendir MSGDIR, $dir || die "Can't open directory $dir: $!\n";
	my @files = grep { /\.msg$/i } readdir(MSGDIR);
	closedir MSGDIR;

	$last = 0;
	foreach $file (@files) {
		if ($file > $last) { $last = $file; }
	}
	return ($last + 1 . ".msg");
}