# Project name:	Meltdown BBS Utilities
# File:		import-txtfile.pl
# Version:	1.0 alpha
# Copyright:	(c) 2004 Petko Bossakov
# Email:	pbosakov@yahoo.com
# WWW:		http://meltdown-bbs.sourceforge.net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

use strict;
use DBI;
use HTML::Entities;
use Getopt::Std;
our ($opt_n, $opt_t, $opt_f, $opt_s, $opt_a);
getopt('ntfsa');

my $dbhost = readini("SQL", "host");
my $dbname = readini("SQL", "dbname");
my $dbuser = readini("SQL", "username");
my $dbpass = readini("SQL", "password");
my $log = readini("Logging", "log");
my ($db, $query, $row);

print "\nMeltdown BBS Utilities textfile import script";
print "\nversion 1.0 alpha";
print "\n\n";

if (!($opt_n) || !($opt_t) || !($opt_f) || !($opt_s) || !($opt_a>0)) {
	print 'Usage: import-txtfile -n filename -f "From Username" -t "To Username" -s "Subject line" -a message_area_num';
	close INI;
	exit;
}

#################
# Open log file #
#################

if ($log) {
	open(LOG, ">>" . readini("Logging", "logfile")) || die "Can't open log file: $!\n";
}

undef $/; # Slurp!
open (TXTFILE, "<$opt_n") || die "Can't open file $opt_n: $!\n";
my $body = <TXTFILE>;
close TXTFILE;

&encode_entities($body, "\"&<>");

###############################
# Connect to the MySQL server #
###############################

$db = DBI->connect("dbi:mysql:database=$dbname;host=$dbhost", $dbuser, $dbpass,
                      { RaiseError => 1, AutoCommit => 1 });

###############
# Add message #
###############

$query = $db->prepare("INSERT INTO msgs(echoid, sender, recipient, subject, datetime, body) VALUES (?,?,?,?,?,?)");
$query->execute($opt_a, $opt_f, $opt_t, $opt_s, time, $body);

# We won't check if the msg area actually exists... 
# it won't hurt if the user entered a wrong one

&writelog("Imported file $opt_n into area $opt_a.");

close INI;
$db->disconnect;

#################################
# readini (section, key, value) #
# reads data from an INI file   #
#################################

sub readini {
	my ($section, $key, $value);
	($section, $key) = @_;
	undef $/;

	open (INI, "mdutil.ini") || die "Can't open mdutil.ini: $!\n";
		while (<INI>) {
			if ( $_ =~ m/^\[$section\]\s*\n([^\n]*\n)*"?$key"?\s*=\s*"?([^\n]+)"?\s*(\n|$)/im ) {
				return $2;
			}
		}
	return "";
}

#######################################################################
# writelog (message)                                                  #
# prints message on screen and writes it to the log file (if defined) #
#######################################################################

sub writelog {
	print LOG localtime(time) . " $_[0]\n" if $log;
	print "$_[0]\n";
}
